from django.conf import settings
from django.db import models
from django.urls import reverse

from hashids import Hashids

from .fields import NaturalSortField


class TimeStampedModelMixin(models.Model):
    "Should be mixed in to all models."
    time_created = models.DateTimeField(auto_now_add=True,
                help_text="The time this item was created in the database.")
    time_modified = models.DateTimeField(auto_now=True,
                help_text="The time this item was last saved to the database.")

    class Meta:
        abstract = True


class SluggedModelMixin(models.Model):
    """
    Adds a `slug` field which is generated from a Hashid of the model's pk.

    `slug` is generated on save, if it doesn't already exist.

    In theory we could use the Hashid'd slug in reverse to get the object's
    pk (e.g. in a view). But we're not relying on that, and simply using
    Hashid as a good method to generate unique, short URL-friendly slugs.
    """
    slug = models.SlugField(max_length=10, null=False, blank=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)

        if not self.slug:
            self.slug = self._generate_slug(self.pk)
            # Don't want to insert again, if that's what was forced:
            kwargs['force_insert'] = False
            self.save(*args, **kwargs)

    def _generate_slug(self, value):
        """
        Generates a slug using a Hashid of `value`.
        """
        # Defaults:
        alphabet = 'abcdefghijkmnopqrstuvwxyz23456789'
        salt = 'Django Spectator'

        if hasattr(settings, 'SPECTATOR_SLUG_ALPHABET'):
            alphabet = settings.SPECTATOR_SLUG_ALPHABET

        if hasattr(settings, 'SPECTATOR_SLUG_SALT'):
            salt = settings.SPECTATOR_SLUG_SALT

        hashids = Hashids(alphabet=alphabet, salt=salt, min_length=5)

        return hashids.encode(value)


class BaseRole(TimeStampedModelMixin, models.Model):
    """
    Base class for linking a Creator to a Book, Event, Movie, etc.

    Child classes should add fields like:

        creator = models.ForeignKey('spectator_core.Creator', blank=False,
                    on_delete=models.CASCADE, related_name='publication_roles')

        publication = models.ForeignKey('spectator_reading.Publication',
                    on_delete=models.CASCADE, related_name='roles')
    """
    role_name = models.CharField(null=False, blank=True, max_length=50,
            help_text="e.g. 'Headliner', 'Support', 'Editor', 'Illustrator', 'Director', etc. Optional.")

    role_order = models.PositiveSmallIntegerField(null=False, blank=False,
            default=1,
            help_text="The order in which the Creators will be listed.")

    class Meta:
        abstract = True
        ordering = ('role_order', 'role_name',)

    def __str__(self):
        if self.role_name:
            return '{} ({})'.format(self.creator, self.role_name)
        else:
            return str(self.creator)


class Creator(TimeStampedModelMixin, SluggedModelMixin, models.Model):
    """
    A person or a group/company/organisation that is responsible for making all
    or part of a book, play, movie, gig, etc.

    Get the things they've worked on:

        creator = Creator.objects.get(pk=1)

        # Just Publication titles:
        for publication in creator.publications.distinct():
            print(publication.title)

        # You can do similar to that to get lists of `events`, `movies` and,
        # `plays` the Creator was involved with.


        # Or Publications including the Creator and their role:
        for role in creator.publication_roles.all():
            print(role.publication, role.creator, role.role_name)

        # Similarly for Event roles:
        for role in creator.event_roles.all():
            print(role.event, role.creator, role.role_name)

        # And for Work roles:
        for role in creator.work_roles.all():
            print(role.work, role.creator, role.role_name)
    """

    KIND_CHOICES = (
        ('individual', 'Individual'),
        ('group', 'Group'),
    )

    name = models.CharField(max_length=255,
            help_text="e.g. 'Douglas Adams' or 'The Long Blondes'.")

    name_sort = NaturalSortField(
        'name', max_length=255, default='',
        help_text="Best for sorting groups. e.g. 'long blondes, the' or 'adams, douglas'.")

    kind = models.CharField(max_length=20, choices=KIND_CHOICES,
                                                        default='individual')

    class Meta:
        ordering = ('name_sort',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('spectator:creators:creator_detail',
                                                    kwargs={'slug':self.slug})

    @property
    def sort_as(self):
        "Used by the NaturalSortField."
        if self.kind == 'individual':
            return 'person'
        else:
            return 'thing'

    def get_works(self):
        "All kinds of Work."
        return self.works.distinct()

    def get_classical_works(self):
        return self.works.filter(kind='classicalwork').distinct()

    def get_dance_pieces(self):
        return self.works.filter(kind='dancepiece').distinct()

    def get_movies(self):
        return self.works.filter(kind='movie').distinct()

    def get_plays(self):
        return self.works.filter(kind='play').distinct()



import logging
import re
import six

from django.db import models

logger = logging.getLogger(__name__)


class UniqueFieldMixin(object):
    """
    Taken from https://github.com/django-extensions/django-extensions/blob/b5404a4a5ed3a5893727b3be3d6a50bc21c534e3/django_extensions/db/fields/__init__.py
    """

    def check_is_bool(self, attrname):
        if not isinstance(getattr(self, attrname), bool):
            raise ValueError("'{}' argument must be True or False".format(attrname))

    @staticmethod
    def _get_fields(model_cls):
        return [
            (f, f.model if f.model != model_cls else None) for f in model_cls._meta.get_fields()
            if not f.is_relation or f.one_to_one or (f.many_to_one and f.related_model)
        ]

    def get_queryset(self, model_cls, slug_field):
        for field, model in self._get_fields(model_cls):
            if model and field == slug_field:
                return model._default_manager.all()
        return model_cls._default_manager.all()

    def find_unique(self, model_instance, field, iterator, *args):
        # exclude the current model instance from the queryset used in finding
        # next valid hash
        queryset = self.get_queryset(model_instance.__class__, field)
        if model_instance.pk:
            queryset = queryset.exclude(pk=model_instance.pk)

        # form a kwarg dict used to impliment any unique_together contraints
        kwargs = {}
        for params in model_instance._meta.unique_together:
            if self.attname in params:
                for param in params:
                    kwargs[param] = getattr(model_instance, param, None)

        new = six.next(iterator)
        kwargs[self.attname] = new
        while not new or queryset.filter(**kwargs):
            new = six.next(iterator)
            kwargs[self.attname] = new
        setattr(model_instance, self.attname, new)
        return new


class NaturalSortField(models.CharField):
    """
    Using the value of another field on the model, make a version that is
    more suitable for sorting.

    If the object has a `sort_as` property and that is set to `person` then
    the string will be treated as if it's a name, i.e. surname put first.

    Either way, this will be done:

    * Stripped of leading/trailing spaces.
    * All lowercase.
    * Integers heavily padded with zeros.

    And, if it's a person:

    * Surnames moved to the start.
        e.g. "David Foster Wallace" becomes "wallace, david foster".
             "John Le Carre" becomes "le carre, john".
             "Daphne du Maurier" becomes "maurier, daphne du".
             "Sir Fred Bloggs Jr" becomes "bloggs, sir fred jr".
             "Prince" is "prince".

    Or, if it's not a person:

    * Articles ("the", "a", etc) moved to the end of the string.
        e.g. "The Long Blondes" becomes "long blondes, the".
             "An Actor Prepares" becomes "actor prepares, an".
             "Le Tigre" becomes "tigre, le".
             "Vol. 2 No. 11, November 2004" becomes
                 "vol. 00000002 no. 00000011, november 00002004".

    So, use like:

        class Author(models.Model):
            name = models.CharField(max_length=255)
            name_sort = NaturalSortField('name',  max_length=255)
            sort_as = 'person'

            class Meta:
                ordering = ('name_sort',)
    """

    description = "A string to allow more human-friendly sorting"

    def __init__(self, for_field, *args, **kwargs):
        """
        for_field - The name of the field to base this field's string on.
                    e.g. 'title' or 'name'.
        """
        self.for_field = for_field
        kwargs.setdefault('db_index', True)
        kwargs.setdefault('editable', False)
        kwargs.setdefault('max_length', 255)
        super(NaturalSortField, self).__init__(**kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        args.append(self.for_field)
        return name, path, args, kwargs

    def pre_save(self, model_instance, add):
        try:
            string = getattr(model_instance, self.for_field)
        except AttributeError as e:
            # During migrations a model being saved does not have access
            # to any of its methods. So we can't access, for example,
            # the @property Event.title_to_sort()
            logger.error("Error in NaturalSortField.pre_save(): {}: The value of the '{}' field will not be changed.".format(e, self.attname))

            string = getattr(model_instance, self.attname)
        else:
            string = string.strip()

            if hasattr(model_instance, 'sort_as') and model_instance.sort_as == 'person':
                string = self.naturalize_person(string)
                # The case of the name is important, so we lowercase afterwards:
                string = string.lower()
            else:
                string = string.lower()
                string = self.naturalize_thing(string)

        return string

    def naturalize_thing(self, string):
        """
        Make a naturalized version of a general string, not a person's name.
        e.g., title of a book, a band's name, etc.

        string -- a lowercase string.
        """

        # Things we want to move to the back of the string:
        articles = [
                        'a', 'an', 'the',
                        'un', 'une', 'le', 'la', 'les', "l'", "l’",
                        'ein', 'eine', 'der', 'die', 'das',
                        'una', 'el', 'los', 'las',
                    ]

        sort_string = string
        parts = string.split(' ')

        if len(parts) > 1 and parts[0] in articles:
            if parts[0] != parts[1]:
                # Don't do this if the name is 'The The' or 'La La Land'.
                # Makes 'long blondes, the':
                sort_string = '{}, {}'.format(' '.join(parts[1:]), parts[0])

        sort_string = self._naturalize_numbers(sort_string)

        return sort_string

    def naturalize_person(self, string):
        """
        Attempt to make a version of the string that has the surname, if any,
        at the start.

        'John, Brown' to 'Brown, John'
        'Sir John Brown Jr' to 'Brown, Sir John Jr'
        'Prince' to 'Prince'

        string -- The string to change.
        """
        suffixes = [
                    'Jr', 'Jr.', 'Sr', 'Sr.',
                    'I', 'II', 'III', 'IV', 'V',
                    ]
        # Add lowercase versions:
        suffixes = suffixes + [s.lower() for s in suffixes]

        # If a name has a capitalised particle in we use that to sort.
        # So 'Le Carre, John' but 'Carre, John le'.
        particles = [
                    'Le', 'La',
                    'Von', 'Van',
                    'Du', 'De',
                    ]

        surname = '' # Smith
        names = ''   # Fred James
        suffix = ''  # Jr

        sort_string = string
        parts = string.split(' ')

        if parts[-1] in suffixes:
            # Remove suffixes entirely, as we'll add them back on the end.
            suffix = parts[-1]
            parts = parts[0:-1] # Remove suffix from parts
            sort_string = ' '.join(parts)

        if len(parts) > 1:

            if parts[-2] in particles:
                # From ['Alan', 'Barry', 'Le', 'Carré']
                # to   ['Alan', 'Barry', 'Le Carré']:
                parts = parts[0:-2] + [ ' '.join(parts[-2:]) ]

            # From 'David Foster Wallace' to 'Wallace, David Foster':
            sort_string = '{}, {}'.format(parts[-1], ' '.join(parts[:-1]))

        if suffix:
            # Add it back on.
            sort_string = '{} {}'.format(sort_string, suffix)

        # In case this name has any numbers in it.
        sort_string = self._naturalize_numbers(sort_string)

        return sort_string

    def _naturalize_numbers(self, string):
        """
        Makes any integers into very zero-padded numbers.
        e.g. '1' becomes '00000001'.
        """

        def naturalize_int_match(match):
            return '%08d' % (int(match.group(0)),)

        string = re.sub(r'\d+', naturalize_int_match, string)

        return string


class PersonNaturalSortField(NaturalSortField):
    pass

class PersonDisplayNaturalSortField(NaturalSortField):
    pass



from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.urls import reverse

from spectator.core.models import BaseRole, Creator, SluggedModelMixin,\
        TimeStampedModelMixin
from . import managers
from spectator.core.fields import NaturalSortField


class PublicationSeries(TimeStampedModelMixin, SluggedModelMixin, models.Model):
    """
    A way to group `Publication`s into series.

    Get its Publications:

        series.publication_set.all()
    """
    title = models.CharField(null=False, blank=False, max_length=255,
            help_text="e.g. 'The London Review of Books'.")

    title_sort = NaturalSortField('title', max_length=255, default='',
            help_text="e.g. 'london review of books, the'.")

    url = models.URLField(null=False, blank=True, max_length=255,
            verbose_name='URL', help_text="e.g. 'https://www.lrb.co.uk/'.")

    class Meta:
        ordering = ('title_sort',)
        verbose_name = 'Publication series'
        verbose_name_plural = 'Publication series'

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('spectator:reading:publicationseries_detail',
                                                        kwargs={'slug':self.slug})


class PublicationRole(BaseRole):
    """
    Linking a creator to a Publication, optionally via their role (e.g.
    'Author', 'Editor', etc.)
    """
    creator = models.ForeignKey('spectator_core.Creator', blank=False,
                    on_delete=models.CASCADE, related_name='publication_roles')

    publication = models.ForeignKey('spectator_reading.Publication',
            on_delete=models.CASCADE, related_name='roles')

    class Meta:
        ordering = ('role_order', 'role_name',)
        verbose_name = 'Publication role'


class Publication(TimeStampedModelMixin, SluggedModelMixin, models.Model):
    """
    Get a Publication's creators:

        publication = Publication.objects.get(pk=1)

        # Just the creators:
        for creator in publication.creators.all():
            print(creator.name)

        # Include their roles:
        for role in publication.roles.all():
            print(role.publication, role.creator, role.role_name)

    Get its readings:

        for reading in publication.reading_set.all():
            print(reading.start_date, reading.end_date)
    """

    KIND_CHOICES = (
        ('book', 'Book'),
        ('periodical', 'Periodical'),
    )

    title = models.CharField(null=False, blank=False, max_length=255,
            help_text="e.g. 'Aurora' or 'Vol. 39 No. 4, 16 February 2017'.")

    title_sort = NaturalSortField('title', max_length=255, default='',
            help_text="e.g. 'clockwork orange, a' or 'world cities, the'.")

    series = models.ForeignKey('spectator_reading.PublicationSeries',
            blank=True, null=True, on_delete=models.SET_NULL)

    kind = models.CharField(max_length=20, choices=KIND_CHOICES,
                                                        default='book')

    official_url = models.URLField(null=False, blank=True, max_length=255,
            verbose_name='Official URL',
            help_text="Official URL for this book/issue.")

    isbn_uk = models.CharField(null=False, blank=True, max_length=20,
            verbose_name='UK ISBN', help_text="e.g. '0356500489'.")

    isbn_us = models.CharField(null=False, blank=True, max_length=20,
            verbose_name='US ISBN', help_text="e.g. '0316098094'.")

    notes_url = models.URLField(null=False, blank=True, max_length=255,
            verbose_name='Notes URL', help_text="URL of your notes/review.")

    creators = models.ManyToManyField('spectator_core.Creator',
                    through='PublicationRole', related_name='publications')

    objects = models.Manager()
    # Publications that are currently being read:
    in_progress_objects = managers.InProgressPublicationsManager()
    # Publications that haven't been started (have no Readings):
    unread_objects = managers.UnreadPublicationsManager()

    class Meta:
        ordering = ('title_sort',)

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('spectator:reading:publication_detail',
                                                        kwargs={'slug':self.slug})

    def get_current_reading(self):
        try:
            return self.reading_set.filter(end_date__isnull=True)[0]
        except IndexError:
            pass

    @property
    def amazon_uk_url(self):
        url = ''
        if self.isbn_uk:
            url = 'https://www.amazon.co.uk/gp/product/{}/'.format(self.isbn_uk)
            if hasattr(settings, 'SPECTATOR_AMAZON') and 'uk' in settings.SPECTATOR_AMAZON:
                url = '{}?tag={}'.format(url, settings.SPECTATOR_AMAZON['uk'])
        return url

    @property
    def amazon_us_url(self):
        url = ''
        if self.isbn_us:
            url = 'https://www.amazon.com/dp/{}/'.format(self.isbn_us)
            if hasattr(settings, 'SPECTATOR_AMAZON') and 'us' in settings.SPECTATOR_AMAZON:
                url = '{}?tag={}'.format(url, settings.SPECTATOR_AMAZON['us'])
        return url

    @property
    def amazon_urls(self):
        urls = []
        if self.isbn_uk:
            urls.append({
                'url': self.amazon_uk_url,
                'name': 'Amazon.co.uk',
                'country': 'UK',
            })
        if self.isbn_us:
            urls.append({
                'url': self.amazon_us_url,
                'name': 'Amazon.com',
                'country': 'USA',
            })
        return urls

    @property
    def has_urls(self):
        "Handy for templates."
        if self.isbn_uk or self.isbn_us or self.official_url or self.notes_url:
            return True
        else:
            return False


class Reading(TimeStampedModelMixin, models.Model):
    """
    A period when a Publication was read.
    """
    # Via https://www.flickr.com/services/api/misc.dates.html
    DATE_GRANULARITIES = (
        # (0, 'Y-m-d H:i:s'),
        (3, 'Y-m-d'),
        (4, 'Y-m'),
        (6, 'Y'),
        # (8, 'Circa...'),
    )

    publication = models.ForeignKey('spectator_reading.Publication',
            null=False, blank=False, on_delete=models.CASCADE)

    start_date = models.DateField(null=True, blank=True)
    start_granularity = models.PositiveSmallIntegerField(null=False,
            blank=False, default=3, choices=DATE_GRANULARITIES)
    end_date = models.DateField(null=True, blank=True)
    end_granularity = models.PositiveSmallIntegerField(null=False,
            blank=False, default=3, choices=DATE_GRANULARITIES)
    is_finished = models.BooleanField(default=False,
            help_text="Did you finish the publication?")

    objects = managers.EndDateAscendingReadingsManager()
    objects_desc = managers.EndDateDescendingReadingsManager()

    def __str__(self):
        return '{} ({} to {})'.format(
                            self.publication, self.start_date, self.end_date)

    def clean(self):
        if self.start_date and self.end_date and self.start_date > self.end_date:
            raise ValidationError(
                    "A Reading's end date can't be before its start date.")



# coding: utf-8
import datetime

from django.core.validators import RegexValidator
from django.db import models
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from spectator.core.models import BaseRole, SluggedModelMixin,\
        TimeStampedModelMixin
from spectator.core.fields import NaturalSortField
from spectator.core.utils import truncate_string


class EventRole(BaseRole):
    """
    Through model for linking a Creator to an Event, optionally via their role
    (e.g. 'Headliner', 'Support', 'Pianist', 'Actor', etc.)

    Every time one of these is saved/deleted a signal re-saves the Event
    in case its `title_sort` needs to change.
    """
    creator = models.ForeignKey('spectator_core.Creator', blank=False,
                        on_delete=models.CASCADE, related_name='event_roles')

    event = models.ForeignKey('spectator_events.Event', on_delete=models.CASCADE,
                                                        related_name='roles')

    class Meta:
        verbose_name = 'event role'
        ordering = ('role_order', 'role_name',)


class Event(TimeStampedModelMixin, SluggedModelMixin, models.Model):
    """
    A thing that happened at a particular venue on a particular date.

    You can get all the Event's Works by doing:

        event = Event.objects.get(pk=1)

        event.works.all()

    But if there are more than one of a kind of Work, they won't necessarily
    be returned in the correct order, as defined in their through model. To
    return them in the correct order you need to use the selection. e.g.:

        event.work_selections.all()

    Or use the shortcut:

        event.get_works()

    Similarly there are shortcuts for specific kinds of work:

        event.get_classical_works()
        event.get_dance_pieces()
        event.get_movies()
        event.get_plays()

    Each item returned will then have a Work object associated, and an order. e.g.:

        selection = event.work_selections.first()
        print(selection.work.title)
        print(selection.order)

    Similarly, to get Creators who worked directly on the Event (as
    opposed to worked on one of its Works), in the correct order:

        event.roles.all()

    And access the Creators and the order like:

        role = event.roles.first()
        print(role.creator.name)
        print(role.role_order)
    """

    KIND_CHOICES = (
        ('concert',     'Classical'),
        ('comedy',      'Comedy'),
        ('dance',       'Dance'),
        ('exhibition',  'Exhibition'),
        ('gig',         'Gig'),
        ('misc',        'Other'),
        ('movie',       'Movie'),
        ('play',        'Theatre'),
    )

    # Mapping keys from KIND_CHOICES to the slugs we'll use in URLs:
    KIND_SLUGS = {
        'comedy':       'comedy',
        'concert':      'concerts',
        'dance':        'dance',
        'exhibition':   'exhibitions',
        'gig':          'gigs',
        'misc':         'misc',
        'movie':        'movies',
        'play':         'plays'
    }

    kind = models.CharField(max_length=20, choices=KIND_CHOICES, blank=False,
        help_text="Used to categorise event. But any kind of Work can be added to any kind of Event.")

    date = models.DateField(null=True, blank=False)

    venue = models.ForeignKey('spectator_events.Venue', null=True, blank=True,
                                                    on_delete=models.CASCADE)

    venue_name = models.CharField(max_length=255, null=False, blank=True,
            help_text="The name of the Venue when this event occurred. If left blank, will be set automatically.")

    title = models.CharField(null=False, blank=True, max_length=255,
            help_text="Optional. e.g., 'Indietracks 2017', 'Radio 1 Roadshow'.")

    title_sort = NaturalSortField('title_to_sort', max_length=255, default='',
            help_text="e.g. 'reading festival, the' or 'drifters, the'.")

    note = models.TextField(null=False, blank=True,
        help_text="Optional. Paragraphs will be surrounded with &lt;p&gt;&lt;/p&gt; tags. HTML allowed.")

    creators = models.ManyToManyField('spectator_core.Creator',
                                through='EventRole', related_name='events')

    works = models.ManyToManyField('spectator_events.Work',
            through='spectator_events.WorkSelection',
            blank=True)

    kind_slug = models.SlugField(null=False, blank=True,
            help_text="Set when the event is saved.")

    class Meta:
        ordering = ['-date',]

    def __str__(self):
        return self.make_title()

    def save(self, *args, **kwargs):
        self.kind_slug = self.KIND_SLUGS[self.kind]

        if self.venue_name == '' and self.venue is not None:
            # Set the venue_name, if it's not already set and there's a Venue.
            self.venue_name = self.venue.name
        elif self.venue is None:
            # Looks like we've removed the Venue, so unset the venue_name.
            self.venue_name = ''

        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('spectator:events:event_detail', kwargs={'slug':self.slug})

    def make_title(self):
        if self.title == '':
            title_start = Event.get_kind_name_plural(self.kind)
            title = '{} #{}'.format(title_start, self.pk)

            # We only need their titles:
            work_titles = [str(sel.work.title) for sel in self.work_selections.all()]

            if len(work_titles) > 0:
                if self.pk:
                    # (If it hasn't been saved it has no works yet.)
                    if len(work_titles) == 1:
                        title = work_titles[0]
                    elif len(work_titles) > 1:
                        title = '{} and {}'.format(
                                ', '.join(work_titles[:-1]), work_titles[-1])
            else:
                # It's like a Gig or Comedy; no works.
                roles = list(self.roles.all())
                if len(roles) == 1:
                    title = str(roles[0].creator.name)
                elif len(roles) == 0:
                    title = 'Event #{}'.format(self.pk)
                else:
                    roles = [r.creator.name for r in roles]
                    # Join with commas but 'and' for the last one:
                    title = '{} and {}'.format(
                                            ', '.join(roles[:-1]), roles[-1])
        else:
            title = self.title

        title = truncate_string(title, chars=255, at_word_boundary=True)

        return title

    def get_kinds():
        """
        Returns a list of the kind values ['gig', 'play', etc] in the order in
        which they're listed in `KIND_CHOICES`.
        """
        return [k for k,v in Event.KIND_CHOICES]

    def get_valid_kind_slugs():
        "Returns a list of the slugs that different kinds of Events can have."
        return list(Event.KIND_SLUGS.values())

    def get_kind_name_plural(kind):
        "e.g. 'Gigs' or 'Movies'."
        if kind == 'comedy':
            return 'Comedy'
        elif kind == 'dance':
            return 'Dance'
        elif kind == 'concert':
            return 'Classical'
        elif kind == 'play':
            return 'Theatre'
        else:
            return '{}s'.format(Event.get_kind_name(kind))

    def get_kind_name(kind):
        return {k:v for (k,v) in Event.KIND_CHOICES}[kind]

    def get_kinds_data():
        """
        Returns a dict of all the data about the kinds, keyed to the kind
        value. e.g:
            {
                'gig': {
                    'name': 'Gig',
                    'slug': 'gigs',
                    'name_plural': 'Gigs',
                },
                # etc
            }
        """
        kinds = {k:{'name':v} for k,v in Event.KIND_CHOICES}
        for k,data in kinds.items():
            kinds[k]['slug'] = Event.KIND_SLUGS[k]
            kinds[k]['name_plural'] = Event.get_kind_name_plural(k)
        return kinds

    def get_works(self):
        return self.work_selections.all()

    def get_classical_works(self):
        return self.work_selections.filter(work__kind='classicalwork')

    def get_dance_pieces(self):
        return self.work_selections.filter(work__kind='dancepiece')

    def get_movies(self):
        return self.work_selections.filter(work__kind='movie')

    def get_plays(self):
        return self.work_selections.filter(work__kind='play')

    @property
    def kind_name(self):
        "e.g. 'Gig' or 'Movie'."
        return {k:v for (k,v) in self.KIND_CHOICES}[self.kind]

    @property
    def kind_name_plural(self):
        "e.g. 'Gigs' or 'Movies'."
        return Event.get_kind_name_plural(self.kind)

    @property
    def title_to_sort(self):
        """
        The string we use to create the title_sort property.
        We want to be able to sort by the event's Creators, if it doesn't
        have a title.
        """
        return self.make_title()


class Work(TimeStampedModelMixin, SluggedModelMixin, models.Model):
    """
    A Classical Work, Dance Piece, Movie or Play.
    Not the occasion on which they were watched (that's an Event).

    Example of getting a Works's creators:

        work = Work.objects.get(pk=1)

        # Just the creators:
        for creator in work.creators.all():
            print(creator.name)

        # Include their roles:
        for role in work.roles.all():
            print(role.work, role.creator, role.role_name)

        # When it's been seen:
        for ev in work.event_set.all():
            print(ev.venue, ev.date)
    """

    KIND_CHOICES = (
        ('classicalwork',  'Classical work'),
        ('dancepiece',     'Dance piece'),
        ('movie',          'Movie'),
        ('play',           'Play'),
    )

    # Mapping keys from KIND_CHOICES to the slugs we'll use in URLs:
    KIND_SLUGS = {
        'classicalwork':    'classical-works',
        'dancepiece':       'dance-pieces',
        'movie':            'movies',
        'play':             'plays',
    }

    YEAR_CHOICES = [(r,r) for r in range(1888, datetime.date.today().year+1)]
    YEAR_CHOICES.insert(0, ('', 'Select…'))

    kind = models.CharField(max_length=20, choices=KIND_CHOICES, blank=False)

    title = models.CharField(null=False, blank=False, max_length=255)

    title_sort = NaturalSortField('title', max_length=255, default='',
            help_text="e.g. 'big piece, a' or 'biggest piece, the'.")

    creators = models.ManyToManyField('spectator_core.Creator',
                through='WorkRole', related_name='works')

    imdb_id = models.CharField(null=False, blank=True, max_length=12,
                    verbose_name='IMDb ID',
                    help_text="Starts with 'tt', e.g. 'tt0100842'.",
                    validators=[
                        RegexValidator(
                            regex='^tt\d{7,10}$',
                            message='IMDb ID should be like "tt1234567"',
                            code='invalid_imdb_id'
                        )
                    ]
                )

    year = models.PositiveSmallIntegerField(null=True, blank=True,
                default=None,
                help_text="Year of release, composition, publication, etc.")

    def __str__(self):
        return self.title

    class Meta:
        ordering = ('title_sort',)
        verbose_name = 'work'

    def get_absolute_url(self):
        kind_slug = self.KIND_SLUGS[self.kind]
        return reverse('spectator:events:work_detail',
                        kwargs={'kind_slug': kind_slug, 'slug': self.slug})

    def get_list_url(self, kind_slug=None):
        """
        Get the list URL for this Work.
        You can also pass a kind_slug in (e.g. 'movies') and it will use that
        instead of the Work's kind_slug. (Why? Useful in views. Or tests of
        views, at least.)
        """
        if kind_slug is None:
            kind_slug = self.KIND_SLUGS[self.kind]
        return reverse('spectator:events:work_list',
                                            kwargs={'kind_slug': kind_slug})

    def get_valid_kind_slugs():
        "Returns a list of the slugs that different kinds of Works can have."
        return list(Work.KIND_SLUGS.values())

    @property
    def imdb_url(self):
        if self.imdb_id:
            return 'http://www.imdb.com/title/{}/'.format(self.imdb_id)
        else:
            return ''


class WorkRole(BaseRole):
    """
    Through model for linking a Creator to a Work, optionally via
    their role (e.g. 'Composer', 'Director'.)
    """
    creator = models.ForeignKey('spectator_core.Creator',
                blank=False,
                on_delete=models.CASCADE, related_name='work_roles')

    work = models.ForeignKey('spectator_events.Work',
                on_delete=models.CASCADE,
                related_name='roles')

    class Meta:
        ordering = ('role_order', 'role_name',)
        verbose_name = 'work role'


class WorkSelection(models.Model):
    """
    Through model for linking a Work to an Event with an order.
    """
    event = models.ForeignKey('spectator_events.Event',
                blank=False,
                on_delete=models.CASCADE,
                related_name='work_selections')

    work = models.ForeignKey('spectator_events.Work',
                blank=False,
                on_delete=models.CASCADE,
                related_name='events')

    order = models.PositiveSmallIntegerField(
                default=1,
                blank=False, null=False,
                help_text="Position on the Event programme." )

    class Meta:
        ordering = ('order',)
        verbose_name = 'work selection'

    def __str__(self):
        return 'Event #{}: {}'.format(self.event.pk, self.work)


class Venue(TimeStampedModelMixin, SluggedModelMixin, models.Model):
    """
    Where an event happens.
    """
    # From https://github.com/SmileyChris/django-countries/blob/master/django_countries/data.py
    # With those marked #changed being, er, changed.
    COUNTRIES = {
        "AF": _("Afghanistan"),
        "AX": _("Åland Islands"),
        "AL": _("Albania"),
        "DZ": _("Algeria"),
        "AS": _("American Samoa"),
        "AD": _("Andorra"),
        "AO": _("Angola"),
        "AI": _("Anguilla"),
        "AQ": _("Antarctica"),
        "AG": _("Antigua and Barbuda"),
        "AR": _("Argentina"),
        "AM": _("Armenia"),
        "AW": _("Aruba"),
        "AU": _("Australia"),
        "AT": _("Austria"),
        "AZ": _("Azerbaijan"),
        "BS": _("Bahamas"),
        "BH": _("Bahrain"),
        "BD": _("Bangladesh"),
        "BB": _("Barbados"),
        "BY": _("Belarus"),
        "BE": _("Belgium"),
        "BZ": _("Belize"),
        "BJ": _("Benin"),
        "BM": _("Bermuda"),
        "BT": _("Bhutan"),
        "BO": _("Bolivia (Plurinational State of)"),
        "BQ": _("Bonaire, Sint Eustatius and Saba"),
        "BA": _("Bosnia and Herzegovina"),
        "BW": _("Botswana"),
        "BV": _("Bouvet Island"),
        "BR": _("Brazil"),
        "IO": _("British Indian Ocean Territory"),
        "BN": _("Brunei Darussalam"),
        "BG": _("Bulgaria"),
        "BF": _("Burkina Faso"),
        "BI": _("Burundi"),
        "CV": _("Cabo Verde"),
        "KH": _("Cambodia"),
        "CM": _("Cameroon"),
        "CA": _("Canada"),
        "KY": _("Cayman Islands"),
        "CF": _("Central African Republic"),
        "TD": _("Chad"),
        "CL": _("Chile"),
        "CN": _("China"),
        "CX": _("Christmas Island"),
        "CC": _("Cocos (Keeling) Islands"),
        "CO": _("Colombia"),
        "KM": _("Comoros"),
        "CD": _("Congo (the Democratic Republic of the)"),
        "CG": _("Congo"),
        "CK": _("Cook Islands"),
        "CR": _("Costa Rica"),
        "CI": _("Côte d'Ivoire"),
        "HR": _("Croatia"),
        "CU": _("Cuba"),
        "CW": _("Curaçao"),
        "CY": _("Cyprus"),
        "CZ": _("Czechia"),
        "DK": _("Denmark"),
        "DJ": _("Djibouti"),
        "DM": _("Dominica"),
        "DO": _("Dominican Republic"),
        "EC": _("Ecuador"),
        "EG": _("Egypt"),
        "SV": _("El Salvador"),
        "GQ": _("Equatorial Guinea"),
        "ER": _("Eritrea"),
        "EE": _("Estonia"),
        "ET": _("Ethiopia"),
        "FK": _("Falkland Islands  [Malvinas]"),
        "FO": _("Faroe Islands"),
        "FJ": _("Fiji"),
        "FI": _("Finland"),
        "FR": _("France"),
        "GF": _("French Guiana"),
        "PF": _("French Polynesia"),
        "TF": _("French Southern Territories"),
        "GA": _("Gabon"),
        "GM": _("Gambia"),
        "GE": _("Georgia"),
        "DE": _("Germany"),
        "GH": _("Ghana"),
        "GI": _("Gibraltar"),
        "GR": _("Greece"),
        "GL": _("Greenland"),
        "GD": _("Grenada"),
        "GP": _("Guadeloupe"),
        "GU": _("Guam"),
        "GT": _("Guatemala"),
        "GG": _("Guernsey"),
        "GN": _("Guinea"),
        "GW": _("Guinea-Bissau"),
        "GY": _("Guyana"),
        "HT": _("Haiti"),
        "HM": _("Heard Island and McDonald Islands"),
        "VA": _("Holy See"),
        "HN": _("Honduras"),
        "HK": _("Hong Kong"),
        "HU": _("Hungary"),
        "IS": _("Iceland"),
        "IN": _("India"),
        "ID": _("Indonesia"),
        "IR": _("Iran (Islamic Republic of)"),
        "IQ": _("Iraq"),
        "IE": _("Ireland"),
        "IM": _("Isle of Man"),
        "IL": _("Israel"),
        "IT": _("Italy"),
        "JM": _("Jamaica"),
        "JP": _("Japan"),
        "JE": _("Jersey"),
        "JO": _("Jordan"),
        "KZ": _("Kazakhstan"),
        "KE": _("Kenya"),
        "KI": _("Kiribati"),
        "KP": _("Korea (the Democratic People's Republic of)"),
        "KR": _("Korea (the Republic of)"),
        "KW": _("Kuwait"),
        "KG": _("Kyrgyzstan"),
        "LA": _("Lao People's Democratic Republic"),
        "LV": _("Latvia"),
        "LB": _("Lebanon"),
        "LS": _("Lesotho"),
        "LR": _("Liberia"),
        "LY": _("Libya"),
        "LI": _("Liechtenstein"),
        "LT": _("Lithuania"),
        "LU": _("Luxembourg"),
        "MO": _("Macao"),
        "MK": _("Macedonia (the former Yugoslav Republic of)"),
        "MG": _("Madagascar"),
        "MW": _("Malawi"),
        "MY": _("Malaysia"),
        "MV": _("Maldives"),
        "ML": _("Mali"),
        "MT": _("Malta"),
        "MH": _("Marshall Islands"),
        "MQ": _("Martinique"),
        "MR": _("Mauritania"),
        "MU": _("Mauritius"),
        "YT": _("Mayotte"),
        "MX": _("Mexico"),
        "FM": _("Micronesia (Federated States of)"),
        "MD": _("Moldova (the Republic of)"),
        "MC": _("Monaco"),
        "MN": _("Mongolia"),
        "ME": _("Montenegro"),
        "MS": _("Montserrat"),
        "MA": _("Morocco"),
        "MZ": _("Mozambique"),
        "MM": _("Myanmar"),
        "NA": _("Namibia"),
        "NR": _("Nauru"),
        "NP": _("Nepal"),
        "NL": _("Netherlands"),
        "NC": _("New Caledonia"),
        "NZ": _("New Zealand"),
        "NI": _("Nicaragua"),
        "NE": _("Niger"),
        "NG": _("Nigeria"),
        "NU": _("Niue"),
        "NF": _("Norfolk Island"),
        "MP": _("Northern Mariana Islands"),
        "NO": _("Norway"),
        "OM": _("Oman"),
        "PK": _("Pakistan"),
        "PW": _("Palau"),
        "PS": _("Palestine, State of"),
        "PA": _("Panama"),
        "PG": _("Papua New Guinea"),
        "PY": _("Paraguay"),
        "PE": _("Peru"),
        "PH": _("Philippines"),
        "PN": _("Pitcairn"),
        "PL": _("Poland"),
        "PT": _("Portugal"),
        "PR": _("Puerto Rico"),
        "QA": _("Qatar"),
        "RE": _("Réunion"),
        "RO": _("Romania"),
        "RU": _("Russian Federation"),
        "RW": _("Rwanda"),
        "BL": _("Saint Barthélemy"),
        "SH": _("Saint Helena, Ascension and Tristan da Cunha"),
        "KN": _("Saint Kitts and Nevis"),
        "LC": _("Saint Lucia"),
        "MF": _("Saint Martin (French part)"),
        "PM": _("Saint Pierre and Miquelon"),
        "VC": _("Saint Vincent and the Grenadines"),
        "WS": _("Samoa"),
        "SM": _("San Marino"),
        "ST": _("Sao Tome and Principe"),
        "SA": _("Saudi Arabia"),
        "SN": _("Senegal"),
        "RS": _("Serbia"),
        "SC": _("Seychelles"),
        "SL": _("Sierra Leone"),
        "SG": _("Singapore"),
        "SX": _("Sint Maarten (Dutch part)"),
        "SK": _("Slovakia"),
        "SI": _("Slovenia"),
        "SB": _("Solomon Islands"),
        "SO": _("Somalia"),
        "ZA": _("South Africa"),
        "GS": _("South Georgia and the South Sandwich Islands"),
        "SS": _("South Sudan"),
        "ES": _("Spain"),
        "LK": _("Sri Lanka"),
        "SD": _("Sudan"),
        "SR": _("Suriname"),
        "SJ": _("Svalbard and Jan Mayen"),
        "SZ": _("Swaziland"),
        "SE": _("Sweden"),
        "CH": _("Switzerland"),
        "SY": _("Syrian Arab Republic"),
        "TW": _("Taiwan (Province of China)"),
        "TJ": _("Tajikistan"),
        "TZ": _("Tanzania, United Republic of"),
        "TH": _("Thailand"),
        "TL": _("Timor-Leste"),
        "TG": _("Togo"),
        "TK": _("Tokelau"),
        "TO": _("Tonga"),
        "TT": _("Trinidad and Tobago"),
        "TN": _("Tunisia"),
        "TR": _("Turkey"),
        "TM": _("Turkmenistan"),
        "TC": _("Turks and Caicos Islands"),
        "TV": _("Tuvalu"),
        "UG": _("Uganda"),
        "UA": _("Ukraine"),
        "AE": _("United Arab Emirates"),
        "GB": _("UK"), #changed
        "UM": _("United States Minor Outlying Islands"),
        "US": _("USA"), #changed
        "UY": _("Uruguay"),
        "UZ": _("Uzbekistan"),
        "VU": _("Vanuatu"),
        "VE": _("Venezuela (Bolivarian Republic of)"),
        "VN": _("Viet Nam"),
        "VG": _("Virgin Islands (British)"),
        "VI": _("Virgin Islands (U.S.)"),
        "WF": _("Wallis and Futuna"),
        "EH": _("Western Sahara"),
        "YE": _("Yemen"),
        "ZM": _("Zambia"),
        "ZW": _("Zimbabwe"),
    }

    COUNTRY_CHOICES = [(k,v) for k,v in COUNTRIES.items()]

    name = models.CharField(null=False, blank=False, max_length=255)

    name_sort = NaturalSortField('name', max_length=255, default='',
            help_text="e.g. 'venue, a' or 'biggest venue, the'.")

    note = models.TextField(null=False, blank=True,
        help_text="Optional. Paragraphs will be surrounded with &lt;p&gt;&lt;/p&gt; tags. HTML allowed.")

    cinema_treasures_id = models.PositiveIntegerField(
        null=True, blank=True,
        help_text='Optional. ID of a cinema at <a href="http://cinematreasures.org/">Cinema Treasures</a>.')

    latitude = models.DecimalField(max_digits=9, decimal_places=6,
                                                        null=True, blank=True)
    longitude = models.DecimalField(max_digits=9, decimal_places=6,
                                                        null=True, blank=True)

    address = models.CharField(null=False, blank=True, max_length=255)

    country = models.CharField(null=False, blank=True, max_length=2,
                    choices=COUNTRY_CHOICES,
                    help_text="The ISO 3166-1 alpha-2 code, e.g. 'GB' or 'FR'")

    class Meta:
        ordering = ['name_sort',]

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('spectator:events:venue_detail', kwargs={'slug':self.slug})

    @property
    def country_name(self):
        if self.country:
            return self.COUNTRIES[self.country]
        else:
            return None

    @property
    def cinema_treasures_url(self):
        if self.cinema_treasures_id is not None:
            return 'http://cinematreasures.org/theaters/{}'.format(
                                                    self.cinema_treasures_id)
        else:
            return ''



from django.db import models
from django.db.models import F, Min


class InProgressPublicationsManager(models.Manager):
    """
    Returns Publications that are currently being read, ordered with the
    most-recently-started last.
    They might have previously been finished.
    """
    def get_queryset(self):
        from .models import Publication
        return super().get_queryset()\
                .filter(reading__start_date__isnull=False,
                        reading__end_date__isnull=True)\
                .annotate(min_start_date=Min('reading__start_date'))\
                .order_by('min_start_date')


class UnreadPublicationsManager(models.Manager):
    """
    Returns Publications that haven't been started (have no Readings).
    """
    def get_queryset(self):
        from .models import Publication
        return super().get_queryset().filter(reading__isnull=True)


class EndDateAscendingReadingsManager(models.Manager):
    """
    Returns Readings in descending end_date order, with Readings that have
    no end_date first.
    Via http://stackoverflow.com/a/15125261/250962
    """
    def get_queryset(self):
        from .models import Reading
        qs = super().get_queryset()
        qs = qs.extra(select={'end_date_null': 'end_date is null'})
        return qs.extra(order_by=['end_date_null', 'end_date'])

class EndDateDescendingReadingsManager(models.Manager):
    """
    Returns Readings in ascending end_date order, with Readings that have
    no end_date last.
    Via http://stackoverflow.com/a/15125261/250962
    """
    def get_queryset(self):
        from .models import Reading
        qs = super().get_queryset()
        qs = qs.extra(select={'end_date_null': 'end_date is null'})
        return qs.extra(order_by=['-end_date_null', '-end_date'])





# Generated by Django 2.0 on 2018-01-31 17:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0024_event_venue_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='venue',
            name='cinema_treasures_id',
            field=models.PositiveIntegerField(blank=True, help_text='Optional. ID of a cinema at <a href="http://cinematreasures.org/">Cinema Treasures</a>.', null=True),
        ),
    ]



from django.contrib.sitemaps import Sitemap

from .models import Event, Venue, Work


class EventSitemap(Sitemap):
    changefreq = 'never'
    priority = 0.5

    def items(self):
        # Exclude movies and plays because they'll have the same URLs as their
        # Movie and Play objects.
        return Event.objects.exclude(kind='movie')\
                            .exclude(kind='play')

    def lastmod(self, obj):
        return obj.time_modified


class VenueSitemap(Sitemap):
    changefreq = 'monthly'
    priority = 0.5

    def items(self):
        return Venue.objects.all()

    def lastmod(self, obj):
        return obj.time_modified


class WorkSitemap(Sitemap):
    changefreq = 'monthly'
    priority = 0.5

    def items(self):
        return Work.objects.all()

    def lastmod(self, obj):
        return obj.time_modified



# Generated by Django 2.0 on 2018-01-26 09:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0015_rename_classical_dance_on_event'),
    ]

    operations = [
        migrations.CreateModel(
            name='MovieSelection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveSmallIntegerField(default=1, help_text='Position on the Event programme.')),
            ],
            options={
                'verbose_name': 'movie selection',
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='PlaySelection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveSmallIntegerField(default=1, help_text='Position on the Event programme.')),
            ],
            options={
                'verbose_name': 'play selection',
                'ordering': ('order',),
            },
        ),
        migrations.AlterField(
            model_name='event',
            name='movie',
            field=models.ForeignKey(blank=True, help_text="Only used if event is of 'Movie' kind.", null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='old_movie', to='spectator_events.Movie'),
        ),
        migrations.AlterField(
            model_name='event',
            name='play',
            field=models.ForeignKey(blank=True, help_text="Only used if event is of 'Play' kind.", null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='old_play', to='spectator_events.Play'),
        ),
        migrations.AddField(
            model_name='playselection',
            name='event',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='play_selections', to='spectator_events.Event'),
        ),
        migrations.AddField(
            model_name='playselection',
            name='play',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='spectator_events.Play'),
        ),
        migrations.AddField(
            model_name='movieselection',
            name='event',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='movie_selections', to='spectator_events.Event'),
        ),
        migrations.AddField(
            model_name='movieselection',
            name='movie',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='spectator_events.Movie'),
        ),
        migrations.AddField(
            model_name='event',
            name='movies',
            field=models.ManyToManyField(blank=True, help_text="Only used if event is of 'Movie' kind.", through='spectator_events.MovieSelection', to='spectator_events.Movie'),
        ),
        migrations.AddField(
            model_name='event',
            name='plays',
            field=models.ManyToManyField(blank=True, help_text="Only used if event is of 'Play' kind.", through='spectator_events.PlaySelection', to='spectator_events.Play'),
        ),
    ]



# Generated by Django 2.0 on 2018-01-02 09:59

from django.db import migrations, models
from django.conf import settings
from hashids import Hashids


def generate_slug(value):
    "A copy of spectator.core.models.SluggedModelMixin._generate_slug()"
    alphabet = 'abcdefghijkmnopqrstuvwxyz23456789'
    salt = 'Django Spectator'

    if hasattr(settings, 'SPECTATOR_SLUG_ALPHABET'):
        alphabet = settings.SPECTATOR_SLUG_ALPHABET

    if hasattr(settings, 'SPECTATOR_SLUG_SALT'):
        salt = settings.SPECTATOR_SLUG_SALT

    hashids = Hashids(alphabet=alphabet, salt=salt, min_length=5)

    return hashids.encode(value)


def set_slug(apps, schema_editor):
    """
    Create a slug for each Creator already in the DB.
    """
    Creator = apps.get_model('spectator_core', 'Creator')

    for c in Creator.objects.all():
        c.slug = generate_slug(c.pk)
        c.save(update_fields=['slug'])


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_core', '0003_set_creator_slug'),
    ]

    operations = [
        migrations.AlterField(
            model_name='creator',
            name='slug',
            field=models.SlugField(blank=True, max_length=10),
        ),
        migrations.RunPython(set_slug),
    ]



from django.core.paginator import InvalidPage
from django.http import Http404
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, ListView, YearArchiveView,\
        TemplateView

from .models import Creator
from .paginator import DiggPaginator
from .apps import spectator_apps

if spectator_apps.is_enabled('events'):
    from spectator.events.models import Event

if spectator_apps.is_enabled('reading'):
    from spectator.reading.models import Publication


class PaginatedListView(ListView):
    """Use this instead of ListView to provide standardised pagination."""
    paginator_class = DiggPaginator
    paginate_by = 50
    page_kwarg = 'p'

    # See spectator.paginator for what these mean:
    paginator_body = 5
    paginator_margin = 2
    paginator_padding = 2
    paginator_tail = 2

    def __init__(self, **kwargs):
        return super().__init__(**kwargs)

    def paginate_queryset(self, queryset, page_size):
        """
        Paginate the queryset, if needed.

        This is EXACTLY the same as the standard ListView.paginate_queryset()
        except for this line:
            page = paginator.page(page_number, softlimit=True)
        Because we want to use the DiggPaginator's softlimit option.
        So that if you're viewing a page of, say, Flickr photos, and you switch
        from viewing by Uploaded Time to viewing by Taken Time, the new
        ordering might have fewer pages. In that case we want to see the final
        page, not a 404. The softlimit does that, but I can't see how to use
        it without copying all of this...
        """
        paginator = self.get_paginator(
            queryset,
            page_size,
            orphans = self.get_paginate_orphans(),
            allow_empty_first_page = self.get_allow_empty(),
            body    = self.paginator_body,
            margin  = self.paginator_margin,
            padding = self.paginator_padding,
            tail    = self.paginator_tail,
        )
        page_kwarg = self.page_kwarg
        page = self.kwargs.get(page_kwarg) or self.request.GET.get(page_kwarg) or 1
        try:
            page_number = int(page)
        except ValueError:
            if page == 'last':
                page_number = paginator.num_pages
            else:
                raise Http404(_("Page is not 'last', nor can it be converted to an int."))
        try:
            page = paginator.page(page_number, softlimit=False)
            return (paginator, page, page.object_list, page.has_other_pages())
        except InvalidPage as e:
            raise Http404(_('Invalid page (%(page_number)s): %(message)s') % {
                'page_number': page_number,
                'message': str(e)
            })


class HomeView(TemplateView):
    template_name = 'spectator_core/home.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if spectator_apps.is_enabled('reading'):
            context['in_progress_publication_list'] = \
                                    Publication.in_progress_objects\
                                    .select_related('series')\
                                    .prefetch_related('roles__creator').all()
        if spectator_apps.is_enabled('events'):
            context['recent_event_list'] = Event.objects\
                                            .select_related('venue')\
                                            .order_by('-date')[:10]
        return context


class CreatorListView(PaginatedListView):
    model = Creator
    creator_kind = 'individual'

    def get(self, request, *args, **kwargs):
        # Are we should 'individual's (default) or 'group's?
        if self.kwargs.get('kind', None) == 'group':
            self.creator_kind = 'group'
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['creator_kind'] = self.creator_kind
        context['individual_count'] = Creator.objects.filter(
                                                    kind='individual').count()
        context['group_count'] = Creator.objects.filter(
                                                    kind='group').count()
        return context

    def get_queryset(self):
        queryset = super().get_queryset()
        queryset = queryset.filter(kind=self.creator_kind)
        return queryset


class CreatorDetailView(DetailView):
    model = Creator





# Generated by Django 2.0 on 2018-02-08 11:26

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import spectator.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_core', '0004_auto_20180102_0959'),
        ('spectator_events', '0025_auto_20180131_1755'),
    ]

    operations = [
        migrations.CreateModel(
            name='Work',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('slug', models.SlugField(blank=True, max_length=10)),
                ('kind', models.CharField(choices=[('classicalwork', 'Classical work'), ('dancepiece', 'Dance piece'), ('movie', 'Movie'), ('play', 'Play')], max_length=20)),
                ('title', models.CharField(max_length=255)),
                ('title_sort', spectator.core.fields.NaturalSortField('title', db_index=True, default='', editable=False, help_text="e.g. 'big piece, a' or 'biggest piece, the'.", max_length=255)),
                ('imdb_id', models.CharField(blank=True, help_text="Starts with 'tt', e.g. 'tt0100842'.", max_length=12, validators=[django.core.validators.RegexValidator(code='invalid_imdb_id', message='IMDb ID should be like "tt1234567"', regex='^tt\\d{7,10}$')], verbose_name='IMDb ID')),
                ('year', models.PositiveSmallIntegerField(blank=True, default=None, help_text='Year of release, composition, publication, etc.', null=True)),
            ],
            options={
                'verbose_name': 'work',
                'ordering': ('title_sort',),
            },
        ),
        migrations.CreateModel(
            name='WorkRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('role_name', models.CharField(blank=True, help_text="e.g. 'Headliner', 'Support', 'Editor', 'Illustrator', 'Director', etc. Optional.", max_length=50)),
                ('role_order', models.PositiveSmallIntegerField(default=1, help_text='The order in which the Creators will be listed.')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='work_roles', to='spectator_core.Creator')),
                ('work', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='roles', to='spectator_events.Work')),
            ],
            options={
                'verbose_name': 'work role',
                'ordering': ('role_order', 'role_name'),
            },
        ),
        migrations.CreateModel(
            name='WorkSelection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveSmallIntegerField(default=1, help_text='Position on the Event programme.')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='work_selections', to='spectator_events.Event')),
                ('work', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='spectator_events.Work')),
            ],
            options={
                'verbose_name': 'work selection',
                'ordering': ('order',),
            },
        ),
        migrations.AddField(
            model_name='work',
            name='creators',
            field=models.ManyToManyField(related_name='works', through='spectator_events.WorkRole', to='spectator_core.Creator'),
        ),
    ]



import factory

from . import models


class IndividualCreatorFactory(factory.DjangoModelFactory):
    "A creator that is an individual person."
    class Meta:
        model = models.Creator

    name = factory.Sequence(lambda n: 'Individual %s' % n)
    kind = 'individual'


class GroupCreatorFactory(factory.DjangoModelFactory):
    "A creator that is a group/organisation/etc."
    class Meta:
        model = models.Creator

    name = factory.Sequence(lambda n: 'Group %s' % n)
    kind = 'group'



# Generated by Django 2.0 on 2018-01-25 17:56

from django.db import migrations


class Migration(migrations.Migration):
    """
    Remove the old ClassicalWork and DancePiece relationships to Events,
    now we've copied their data to the new version with a through model.
    """

    dependencies = [
        ('spectator_events', '0013_copy_classical_and_dance_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='event',
            name='classicalworks',
        ),
        migrations.RemoveField(
            model_name='event',
            name='dancepieces',
        ),
    ]



# Generated by Django 2.0 on 2018-02-08 16:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0032_recreate_work_slugs'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='classicalwork',
            name='creators',
        ),
        migrations.RemoveField(
            model_name='classicalworkrole',
            name='classical_work',
        ),
        migrations.RemoveField(
            model_name='classicalworkrole',
            name='creator',
        ),
        migrations.RemoveField(
            model_name='classicalworkselection',
            name='event',
        ),
        migrations.RemoveField(
            model_name='classicalworkselection',
            name='work',
        ),
        migrations.RemoveField(
            model_name='dancepiece',
            name='creators',
        ),
        migrations.RemoveField(
            model_name='dancepiecerole',
            name='creator',
        ),
        migrations.RemoveField(
            model_name='dancepiecerole',
            name='dance_piece',
        ),
        migrations.RemoveField(
            model_name='dancepieceselection',
            name='event',
        ),
        migrations.RemoveField(
            model_name='dancepieceselection',
            name='work',
        ),
        migrations.RemoveField(
            model_name='movie',
            name='creators',
        ),
        migrations.RemoveField(
            model_name='movierole',
            name='creator',
        ),
        migrations.RemoveField(
            model_name='movierole',
            name='movie',
        ),
        migrations.RemoveField(
            model_name='movieselection',
            name='event',
        ),
        migrations.RemoveField(
            model_name='movieselection',
            name='work',
        ),
        migrations.RemoveField(
            model_name='play',
            name='creators',
        ),
        migrations.RemoveField(
            model_name='playrole',
            name='creator',
        ),
        migrations.RemoveField(
            model_name='playrole',
            name='play',
        ),
        migrations.RemoveField(
            model_name='playselection',
            name='event',
        ),
        migrations.RemoveField(
            model_name='playselection',
            name='work',
        ),
        migrations.AlterField(
            model_name='event',
            name='kind',
            field=models.CharField(choices=[('concert', 'Classical'), ('comedy', 'Comedy'), ('dance', 'Dance'), ('exhibition', 'Exhibition'), ('gig', 'Gig'), ('misc', 'Other'), ('movie', 'Movie'), ('play', 'Theatre')], max_length=20),
        ),
        migrations.DeleteModel(
            name='ClassicalWork',
        ),
        migrations.DeleteModel(
            name='ClassicalWorkRole',
        ),
        migrations.DeleteModel(
            name='ClassicalWorkSelection',
        ),
        migrations.DeleteModel(
            name='DancePiece',
        ),
        migrations.DeleteModel(
            name='DancePieceRole',
        ),
        migrations.DeleteModel(
            name='DancePieceSelection',
        ),
        migrations.DeleteModel(
            name='Movie',
        ),
        migrations.DeleteModel(
            name='MovieRole',
        ),
        migrations.DeleteModel(
            name='MovieSelection',
        ),
        migrations.DeleteModel(
            name='Play',
        ),
        migrations.DeleteModel(
            name='PlayRole',
        ),
        migrations.DeleteModel(
            name='PlaySelection',
        ),
    ]



# Generated by Django 2.0 on 2018-02-08 11:32

from django.db import migrations


def forwards(apps, schema_editor):
    """
    Change all ClassicalWork objects into Work objects, and their associated
    data into WorkRole and WorkSelection models, then delete the ClassicalWork.
    """
    ClassicalWork = apps.get_model('spectator_events', 'ClassicalWork')
    Work = apps.get_model('spectator_events', 'Work')
    WorkRole = apps.get_model('spectator_events', 'WorkRole')
    WorkSelection = apps.get_model('spectator_events', 'WorkSelection')

    for cw in ClassicalWork.objects.all():

        work = Work.objects.create(
            kind='classicalwork',
            title=cw.title,
            title_sort=cw.title_sort
        )

        for role in cw.roles.all():
            WorkRole.objects.create(
                creator=role.creator,
                work=work,
                role_name=role.role_name,
                role_order=role.role_order
            )

        for selection in cw.events.all():
            WorkSelection.objects.create(
                event=selection.event,
                work=work,
                order=selection.order
            )

        cw.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0026_auto_20180208_1126'),
    ]

    operations = [
        migrations.RunPython(forwards),
    ]



# Generated by Django 2.0 on 2018-01-25 13:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0010_auto_20180118_0906'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='classicalwork',
            options={'ordering': ('title_sort',), 'verbose_name': 'classical work'},
        ),
        migrations.AlterModelOptions(
            name='classicalworkrole',
            options={'ordering': ('role_order', 'role_name'), 'verbose_name': 'classical work role'},
        ),
        migrations.AlterModelOptions(
            name='dancepiece',
            options={'ordering': ('title_sort',), 'verbose_name': 'dance piece'},
        ),
        migrations.AlterModelOptions(
            name='dancepiecerole',
            options={'ordering': ('role_order', 'role_name'), 'verbose_name': 'dance piece role'},
        ),
        migrations.AlterModelOptions(
            name='eventrole',
            options={'ordering': ('role_order', 'role_name'), 'verbose_name': 'event role'},
        ),
        migrations.AlterModelOptions(
            name='movierole',
            options={'ordering': ('role_order', 'role_name'), 'verbose_name': 'movie role'},
        ),
        migrations.AlterModelOptions(
            name='playrole',
            options={'ordering': ('role_order', 'role_name'), 'verbose_name': 'play role'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-17 17:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import spectator.core.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('spectator_core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Publication',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('title', models.CharField(help_text="e.g. 'Aurora' or 'Vol. 39 No. 4, 16 February 2017'.", max_length=255)),
                ('title_sort', spectator.core.fields.NaturalSortField('title', db_index=True, default='', editable=False, help_text="e.g. 'clockwork orange, a' or 'world cities, the'.", max_length=255)),
                ('kind', models.CharField(choices=[('book', 'Book'), ('periodical', 'Periodical')], default='book', max_length=20)),
                ('official_url', models.URLField(blank=True, help_text='Official URL for this book/issue.', max_length=255, verbose_name='Official URL')),
                ('isbn_uk', models.CharField(blank=True, help_text="e.g. '0356500489'.", max_length=20, verbose_name='UK ISBN')),
                ('isbn_us', models.CharField(blank=True, help_text="e.g. '0316098094'.", max_length=20, verbose_name='US ISBN')),
                ('notes_url', models.URLField(blank=True, help_text='URL of your notes/review.', max_length=255, verbose_name='Notes URL')),
            ],
            options={
                'ordering': ('title_sort',),
            },
        ),
        migrations.CreateModel(
            name='PublicationRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('role_name', models.CharField(blank=True, help_text="e.g. 'Headliner', 'Support', 'Editor', 'Illustrator', 'Director', etc. Optional.", max_length=50)),
                ('role_order', models.PositiveSmallIntegerField(default=1, help_text='The order in which the Creators will be listed.')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='publication_roles', to='spectator_core.Creator')),
                ('publication', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='roles', to='spectator_reading.Publication')),
            ],
            options={
                'ordering': ('role_order', 'role_name'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PublicationSeries',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('title', models.CharField(help_text="e.g. 'The London Review of Books'.", max_length=255)),
                ('title_sort', spectator.core.fields.NaturalSortField('title', db_index=True, default='', editable=False, help_text="e.g. 'london review of books, the'.", max_length=255)),
                ('url', models.URLField(blank=True, help_text="e.g. 'https://www.lrb.co.uk/'.", max_length=255, verbose_name='URL')),
            ],
            options={
                'verbose_name_plural': 'Publication series',
                'ordering': ('title_sort',),
            },
        ),
        migrations.CreateModel(
            name='Reading',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('start_date', models.DateField(blank=True, null=True)),
                ('start_granularity', models.PositiveSmallIntegerField(choices=[(3, 'Y-m-d'), (4, 'Y-m'), (6, 'Y')], default=3)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('end_granularity', models.PositiveSmallIntegerField(choices=[(3, 'Y-m-d'), (4, 'Y-m'), (6, 'Y')], default=3)),
                ('is_finished', models.BooleanField(default=False, help_text='Did you finish the publication?')),
                ('publication', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spectator_reading.Publication')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='publication',
            name='creators',
            field=models.ManyToManyField(related_name='publications', through='spectator_reading.PublicationRole', to='spectator_core.Creator'),
        ),
        migrations.AddField(
            model_name='publication',
            name='series',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='spectator_reading.PublicationSeries'),
        ),
    ]



# Generated by Django 2.0 on 2018-02-08 14:17

from django.conf import settings
from django.db import migrations

from hashids import Hashids


def generate_slug(value):
    """
    Generates a slug using a Hashid of `value`.

    Taken from spectator_core.models.SluggedModelMixin
    """
    # Defaults:
    alphabet = 'abcdefghijkmnopqrstuvwxyz23456789'
    salt = 'Django Spectator'

    if hasattr(settings, 'SPECTATOR_SLUG_ALPHABET'):
        alphabet = settings.SPECTATOR_SLUG_ALPHABET

    if hasattr(settings, 'SPECTATOR_SLUG_SALT'):
        salt = settings.SPECTATOR_SLUG_SALT

    hashids = Hashids(alphabet=alphabet, salt=salt, min_length=5)

    return hashids.encode(value)


def forwards(apps, schema_editor):
    """
    Re-save all the Works because something earlier didn't create their slugs.
    """
    Work = apps.get_model('spectator_events', 'Work')

    for work in Work.objects.all():
        if not work.slug:
            work.slug = generate_slug(work.pk)
            work.save()


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0031_auto_20180208_1412'),
    ]

    operations = [
        migrations.RunPython(forwards),
    ]



from django.contrib import admin

from .models import Publication, PublicationRole, PublicationSeries, Reading


class ReadingInline(admin.TabularInline):
    model = Reading
    fields = ('publication', 'start_date', 'end_date', 'is_finished',
                        'start_granularity', 'end_granularity',)
    raw_id_fields = ('publication',)
    extra = 1


class PublicationRoleInline(admin.TabularInline):
    model = PublicationRole
    fields = ( 'creator', 'role_name', 'role_order',)
    raw_id_fields = ('creator',)
    extra = 1


@admin.register(PublicationSeries)
class PublicationSeriesAdmin(admin.ModelAdmin):
    list_display = ('title',)

    fieldsets = (
        (None, {
            'fields': ('title', 'title_sort', 'slug', 'url', )
        }),
        ('Times', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('title_sort', 'slug', 'time_created', 'time_modified',)



class ReadingsListFilter(admin.SimpleListFilter):
    """
    Add filters for publications to list 'Unread' and 'In-progress' publications.
    """
    title = ('reading')
    parameter_name = 'readings'

    def lookups(self, request, model_admin):
        return (
            ('in-progress', ('In progress')),
            ('unread', ('Unread')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'in-progress':
            return queryset.filter(reading__start_date__isnull=False,
                                   reading__end_date__isnull=True)

        if self.value() == 'unread':
            return queryset.filter(reading__isnull=True)



@admin.register(Publication)
class PublicationAdmin(admin.ModelAdmin):
    list_display = ('title', 'kind', 'show_creators', 'series', )
    list_filter = (ReadingsListFilter, 'kind', 'series', )
    search_fields = ('title',)
    list_select_related = ('series',)

    fieldsets = (
        (None, {
            'fields': ( 'title', 'title_sort', 'slug', 'kind', 'series',
                        'isbn_uk', 'isbn_us',
                        'official_url', 'notes_url', )
        }),
        ('Times', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    radio_fields = {'kind': admin.HORIZONTAL}
    readonly_fields = ('title_sort', 'slug', 'time_created', 'time_modified',)

    inlines = [ PublicationRoleInline, ReadingInline, ]

    def show_creators(self, instance):
        names = [ str(r.creator) for r in instance.roles.all() ]
        if names:
            return ', '.join(names)
        else:
            return '-'
    show_creators.short_description = 'Creators'



# Generated by Django 2.0 on 2018-01-29 17:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0020_venue_note'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='venue',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='spectator_events.Venue'),
        ),
    ]



import factory

from . import models
from spectator.core.factories import IndividualCreatorFactory


class VenueFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Venue

    name = factory.Sequence(lambda n: 'Venue %s' % n)


class WorkFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Work

    title = factory.Sequence(lambda n: 'Work %s' % n)

class ClassicalWorkFactory(WorkFactory):
    kind = 'classicalwork'
    title = factory.Sequence(lambda n: 'Classical Work %s' % n)

class DancePieceFactory(WorkFactory):
    kind = 'dancepiece'
    title = factory.Sequence(lambda n: 'Dance Piece %s' % n)

class MovieFactory(WorkFactory):
    kind = 'movie'
    title = factory.Sequence(lambda n: 'Movie %s' % n)

class PlayFactory(WorkFactory):
    kind = 'play'
    title = factory.Sequence(lambda n: 'Play %s' % n)


class WorkRoleFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.WorkRole

    role_name = factory.Sequence(lambda n: 'Role %s' % n)
    creator = factory.SubFactory(IndividualCreatorFactory)
    work = factory.SubFactory(WorkFactory)


class EventFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Event

    title = factory.Sequence(lambda n: 'Event %s' % n)
    venue = factory.SubFactory(VenueFactory)


class ComedyEventFactory(EventFactory):
    kind = 'comedy'

class ConcertEventFactory(EventFactory):
    kind = 'concert'

class DanceEventFactory(EventFactory):
    kind = 'dance'

class ExhibitionEventFactory(EventFactory):
    kind = 'exhibition'

class GigEventFactory(EventFactory):
    kind = 'gig'

class MiscEventFactory(EventFactory):
    kind = 'misc'

class MovieEventFactory(EventFactory):
    kind = 'movie'

class PlayEventFactory(EventFactory):
    kind = 'play'


class EventRoleFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.EventRole

    role_name = factory.Sequence(lambda n: 'Role %s' % n)
    creator = factory.SubFactory(IndividualCreatorFactory)
    event = factory.SubFactory(EventFactory)


class WorkSelectionFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.WorkSelection

    event = factory.SubFactory(EventFactory)
    work = factory.SubFactory(WorkFactory)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-01 16:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_reading', '0002_publicationseries_slug'),
    ]

    operations = [
        migrations.AddField(
            model_name='publication',
            name='slug',
            field=models.SlugField(blank=True, default='a', max_length=10),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.0 on 2018-01-25 17:57

from django.db import migrations, models


class Migration(migrations.Migration):
    """
    Now we've deleted the old classicalworks and dancepieces fields,
    rename the new ones to be the same as the old.
    """

    dependencies = [
        ('spectator_events', '0014_remove_old_classical_dance'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='event',
            name='classicalworks2',
        ),
        migrations.RemoveField(
            model_name='event',
            name='dancepieces2',
        ),
        migrations.AddField(
            model_name='event',
            name='classicalworks',
            field=models.ManyToManyField(blank=True, help_text="Only used if event is of 'Classical Concert' kind.", through='spectator_events.ClassicalWorkSelection', to='spectator_events.ClassicalWork'),
        ),
        migrations.AddField(
            model_name='event',
            name='dancepieces',
            field=models.ManyToManyField(blank=True, help_text="Only used if event is of 'Dance' kind.", through='spectator_events.DancePieceSelection', to='spectator_events.DancePiece'),
        ),
    ]



from django import forms
from django.conf import settings
from django.contrib import admin
from django.core.exceptions import ValidationError
from django.db.models import Count
from django.templatetags.l10n import unlocalize

from .models import (
    Event, EventRole,
    Work, WorkRole, WorkSelection,
    Venue
)


# INLINES

class EventRoleInline(admin.TabularInline):
    model = EventRole
    fields = ( 'creator', 'role_name', 'role_order',)
    raw_id_fields = ('creator',)
    extra = 0

class WorkRoleInline(admin.TabularInline):
    model = WorkRole
    fields = ( 'creator', 'role_name', 'role_order',)
    raw_id_fields = ('creator',)
    extra = 0

class WorkSelectionInline(admin.TabularInline):
    model = WorkSelection
    fields = ('work', 'order',)
    raw_id_fields = ('work',)
    extra = 0



# MODEL ADMINS.

@admin.register(Event)
class EventAdmin(admin.ModelAdmin):

    list_display = ('__str__', 'date', 'kind_name', 'venue',)
    list_filter = ('kind', 'date',)
    search_fields = ('title',)

    fieldsets = (
        (None, {
            'fields': ( 'kind', 'date', 'venue', 'venue_name',
                        'title', 'title_sort', 'slug', 'note',)
        }),
        ('Times', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    raw_id_fields = ('venue',)
    readonly_fields = ('title_sort', 'slug', 'time_created', 'time_modified',)

    inlines = [WorkSelectionInline, EventRoleInline, ]

    def save_related(self, request, form, formsets, change):
        """
        When the Admin saves the Event, the related M2M things, like Movies,
        haven't yet been saved. So when the Event's title_sort field has
        nothing to go on.

        So, cheekily, after the parent's save_related() method has saved
        all the M2M fields, we save the Event again. This time its
        title_sort will be set correctly.
        """
        super().save_related(request, form, formsets, change)
        form.instance.save()


@admin.register(Work)
class WorkAdmin(admin.ModelAdmin):
    list_display = ('title', 'kind', 'tidy_year',)
    search_fields = ('title',)
    list_filter = ('kind', 'year',)

    fieldsets = (
        (None, {
            'fields': ( 'kind', 'title', 'title_sort', 'slug',
                        'year', 'imdb_id',)
        }),
        ('Times', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('title_sort', 'slug', 'time_created', 'time_modified',)
    inlines = [ WorkRoleInline, ]

    def tidy_year(self, obj):
        "Stop the year appearing like '2,018' when USE_THOUSAND_SEPARATOR=True"
        if obj.year:
            return unlocalize(obj.year)
        else:
            return '-'
    tidy_year.short_description = 'Year'




class CountryListFilter(admin.SimpleListFilter):
    """
    For filtering Venues by country.

    Used so that we only list countries that have at least one Venue assigned
    to them.

    Otherwise we have a very long list of countries, most of which are unused.
    """
    title = 'Country'

    parameter_name = 'country'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples like:

            [
                ('AU', 'Australia'),
                ('GB', 'UK'),
                ('US', 'USA'),
            ]

        One for each country that has at least one Venue.
        Sorted by the label names.
        """
        list_of_countries = []

        # We don't need the country_count but we need to annotate them in order
        # to group the results.
        qs = Venue.objects.exclude(country='') \
                            .values('country') \
                            .annotate(country_count=Count('country')) \
                            .order_by('country')
        for obj in qs:
            country = obj['country']
            list_of_countries.append(
                (country, Venue.COUNTRIES[country])
            )

        return sorted(list_of_countries, key=lambda c: c[1])

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value():
            return queryset.filter(country=self.value())
        else:
            return queryset


@admin.register(Venue)
class VenueAdmin(admin.ModelAdmin):
    list_display = ('name', 'address', 'country',)
    list_filter = (CountryListFilter,)
    search_fields = ('name',)

    fieldsets = (
        (None, {
            'fields': ( 'name', 'name_sort', 'note', 'cinema_treasures_id',
                        'latitude', 'longitude',
                        'address', 'country',)
        }),
        ('Times', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('name_sort', 'time_created', 'time_modified',)

    class Media:
        if hasattr(settings, 'SPECTATOR_GOOGLE_MAPS_API_KEY') and settings.SPECTATOR_GOOGLE_MAPS_API_KEY:
            css = {
                'all': ('css/admin/location_picker.css',),
            }
            js = (
                'https://maps.googleapis.com/maps/api/js?key={}'.format(settings.SPECTATOR_GOOGLE_MAPS_API_KEY),
                'js/admin/location_picker.js',
            )



from django.contrib.sitemaps import Sitemap

from .models import Publication, PublicationSeries


class PublicationSitemap(Sitemap):
    changefreq = 'yearly'
    priority = 0.5

    def items(self):
        return Publication.objects.all()

    def lastmod(self, obj):
        return obj.time_modified


class PublicationSeriesSitemap(Sitemap):
    changefreq = 'monthly'
    priority = 0.5

    def items(self):
        return PublicationSeries.objects.all()

    def lastmod(self, obj):
        return obj.time_modified




# Generated by Django 2.0 on 2018-01-25 17:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0011_auto_20180125_1348'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClassicalWorkSelection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveSmallIntegerField(default=1, help_text='Position on the Event programme.')),
                ('classical_work', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='spectator_events.ClassicalWork')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='classical_work_selections', to='spectator_events.Event')),
            ],
            options={
                'verbose_name': 'classical work selection',
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='DancePieceSelection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveSmallIntegerField(default=1, help_text='Position on the Event programme.')),
                ('dance_piece', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='spectator_events.DancePiece')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dance_piece_selections', to='spectator_events.Event')),
            ],
            options={
                'verbose_name': 'dance piece selection',
                'ordering': ('order',),
            },
        ),
        migrations.AddField(
            model_name='event',
            name='classicalworks2',
            field=models.ManyToManyField(blank=True, help_text="Only used if event is of 'Classical Concert' kind.", related_name='event_set_2', through='spectator_events.ClassicalWorkSelection', to='spectator_events.ClassicalWork'),
        ),
        migrations.AddField(
            model_name='event',
            name='dancepieces2',
            field=models.ManyToManyField(blank=True, help_text="Only used if event is of 'Dance' kind.", related_name='event_set_2', through='spectator_events.DancePieceSelection', to='spectator_events.DancePiece'),
        ),
    ]



from django.conf.urls import url

from . import views
from .models import Event, Work


app_name = 'events'

# Will be like 'movies|plays|concerts' etc:
event_kind_slugs = '|'.join(Event.get_valid_kind_slugs())

# Will be like 'classical-works|dance-pieces' etc:
work_kind_slugs = '|'.join(Work.get_valid_kind_slugs())

urlpatterns = [
    url(
        regex=r"^$",
        view=views.EventListView.as_view(),
        name='home'
    ),
    url(
        regex=r"^types/(?P<kind_slug>{})/$".format(event_kind_slugs),
        view=views.EventListView.as_view(),
        name='event_list'
    ),

    url(
        regex=r"^venues/$",
        view=views.VenueListView.as_view(),
        name='venue_list',
    ),
    url(
        regex=r"^venues/(?P<slug>[\w-]+)/$",
        view=views.VenueDetailView.as_view(),
        name='venue_detail'
    ),

    url(
        regex=r"^(?P<year>[0-9]{4})/$",
        view=views.EventYearArchiveView.as_view(),
        name='event_year_archive'
    ),

    url(
        regex=r"^(?P<kind_slug>{})/$".format(work_kind_slugs),
        view=views.WorkListView.as_view(),
        name='work_list'
    ),
    url(
        regex=r"^(?P<kind_slug>{})/(?P<slug>[\w-]+)/$".format(work_kind_slugs),
        view=views.WorkDetailView.as_view(),
        name='work_detail'
    ),

    url(
        regex=r"^(?P<slug>[\w-]+)/$",
        view=views.EventDetailView.as_view(),
        name='event_detail'
    ),

]



# Generated by Django 2.0 on 2018-01-25 13:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_reading', '0004_slugs_20180102_1153'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='publicationrole',
            options={'ordering': ('role_order', 'role_name'), 'verbose_name': 'Publication role'},
        ),
        migrations.AlterModelOptions(
            name='publicationseries',
            options={'ordering': ('title_sort',), 'verbose_name': 'Publication series', 'verbose_name_plural': 'Publication series'},
        ),
    ]



# Generated by Django 2.0 on 2018-01-25 17:42

from django.db import migrations


def forwards(apps, schema_editor):
    """
    Copy the ClassicalWork and DancePiece data to use the new through models.
    """
    Event = apps.get_model('spectator_events', 'Event')
    ClassicalWorkSelection = apps.get_model(
                                'spectator_events', 'ClassicalWorkSelection')
    DancePieceSelection = apps.get_model(
                                'spectator_events', 'DancePieceSelection')

    for event in Event.objects.all():

        for work in event.classicalworks.all():
            selection = ClassicalWorkSelection(
                            classical_work=work,
                            event=event)
            selection.save()

        for piece in event.dancepieces.all():
            selection = DancePieceSelection(
                            dance_piece=piece,
                            event=event)
            selection.save()


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0012_add_classical_and_dance_through_models'),
    ]

    operations = [
        migrations.RunPython(forwards),
    ]



# Generated by Django 2.0 on 2018-01-31 15:37

from django.db import migrations, models



def forwards(apps, schema_editor):
    """
    Set the venue_name field of all Events that have a Venue.
    """
    Event = apps.get_model('spectator_events', 'Event')

    for event in Event.objects.all():
        if event.venue is not None:
            event.venue_name = event.venue.name
            event.save()



class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0023_venue_cinema_treasures_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='venue_name',
            field=models.CharField(blank=True, help_text='The name of the Venue when this event occurred. If left blank, will be set automatically.', max_length=255),
        ),
        migrations.RunPython(forwards),
    ]



# Generated by Django 2.0 on 2018-01-27 16:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0018_remove_old_movie_play_fields'),
    ]

    operations = [
        migrations.RenameField(
            model_name='classicalworkselection',
            old_name='classical_work',
            new_name='work',
        ),
        migrations.RenameField(
            model_name='dancepieceselection',
            old_name='dance_piece',
            new_name='work',
        ),
        migrations.RenameField(
            model_name='movieselection',
            old_name='movie',
            new_name='work',
        ),
        migrations.RenameField(
            model_name='playselection',
            old_name='play',
            new_name='work',
        ),
    ]



from django.utils.html import strip_tags
from django.utils.text import Truncator

def truncate_string(text, strip_html=True, chars=255, truncate='…', at_word_boundary=False):
    """Truncate a string to a certain length, removing line breaks and mutliple
    spaces, optionally removing HTML, and appending a 'truncate' string.

    Keyword arguments:
    strip_html -- boolean.
    chars -- Number of characters to return.
    at_word_boundary -- Only truncate at a word boundary, which will probably
        result in a string shorter than chars.
    truncate -- String to add to the end.
    """
    if strip_html:
        text = strip_tags(text)
    text = text.replace('\n', ' ').replace('\r', '')
    text = ' '.join(text.split())
    if at_word_boundary:
        if len(text) > chars:
            text = text[:chars].rsplit(' ', 1)[0] + truncate
    else:
        text = Truncator(text).chars(chars, html=False, truncate=truncate)
    return text



# Generated by Django 2.0 on 2018-02-08 16:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0033_auto_20180208_1613'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='kind',
            field=models.CharField(choices=[('concert', 'Classical'), ('comedy', 'Comedy'), ('dance', 'Dance'), ('exhibition', 'Exhibition'), ('gig', 'Gig'), ('misc', 'Other'), ('movie', 'Movie'), ('play', 'Theatre')], help_text='Used to categorise event. But any kind of Work can be added to any kind of Event.', max_length=20),
        ),
    ]



# Generated by Django 2.0 on 2018-01-26 09:16

from django.db import migrations


class Migration(migrations.Migration):
    """
    Now we've copied their data to the new movies and plays m2m fields,
    we can remove the old movie and play fk fields.
    """

    dependencies = [
        ('spectator_events', '0017_copy_movies_and_plays_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='event',
            name='movie',
        ),
        migrations.RemoveField(
            model_name='event',
            name='play',
        ),
    ]



from django import template
from django.db.models import Q
from django.utils.html import format_html

from ..models import Publication, Reading

register = template.Library()


@register.simple_tag
def in_progress_publications():
    """
    Returns a QuerySet of any Publications that are currently being read.
    """
    return Publication.in_progress_objects\
                        .select_related('series')\
                        .prefetch_related('roles__creator')\
                        .order_by('time_created')


@register.inclusion_tag('spectator_reading/includes/card_publications.html')
def in_progress_publications_card():
    """
    Displays Publications that are currently being read.
    """
    return {
            'card_title': 'Currently reading',
            'publication_list': in_progress_publications(),
            }


@register.simple_tag
def day_publications(date):
    """
    Returns a QuerySet of Publications that were being read on `date`.
    `date` is a date tobject.
    """
    readings = Reading.objects \
                        .filter(start_date__lte=date) \
                        .filter(
                            Q(end_date__gte=date)
                            |
                            Q(end_date__isnull=True)
                        )
    if readings:
        return Publication.objects.filter(reading__in=readings) \
                        .select_related('series') \
                        .prefetch_related('roles__creator')
    else:
        return Publication.objects.none()


@register.inclusion_tag('spectator_reading/includes/card_publications.html')
def day_publications_card(date):
    """
    Displays Publications that were being read on `date`.
    `date` is a date tobject.
    """
    card_title = 'Reading on {}'.format(date.strftime('%-d %b %Y'))
    return {
            'card_title': card_title,
            'publication_list': day_publications(date=date),
            }


@register.simple_tag
def reading_years():
    """
    Returns a QuerySet of date objects, one for each year in which there are
    Readings.
    """
    return Reading.objects.dates('end_date', 'year')


@register.inclusion_tag('spectator_reading/includes/card_years.html')
def reading_years_card(current_year=None):
    """
    Displays the years in which there are Readings.
    Each one is linked to the year archive page.
    current_year is a date object; this year won't be linked.
    """
    return {
            'current_year': current_year,
            'years': reading_years(),
            }


@register.simple_tag
def reading_dates(reading):
    """
    Given a Reading, with start and end dates and granularities[1] it returns
    an HTML string representing that period. eg:

        * '1–6 Feb 2017'
        * '1 Feb to 3 Mar 2017'
        * 'Feb 2017 to Mar 2018'
        * '2017-2018'
    etc.

    [1] https://www.flickr.com/services/api/misc.dates.html
    """

    # 3 Sep 2017
    full_format = '<time datetime="%Y-%m-%d">%-d&nbsp;%b&nbsp;%Y</time>'
    # Sep 2017
    month_year_format = '<time datetime="%Y-%m">%b&nbsp;%Y</time>'
    # 2017
    year_format = '<time datetime="%Y">%Y</time>'
    # 3
    day_format = '<time datetime="%Y-%m-%d">%-d</time>'
    # 3 Sep
    day_month_format = '<time datetime="%Y-%m-%d">%-d&nbsp;%b</time>'
    # Sep
    month_format = '<time datetime="%Y-%m">%b</time>'

    # For brevity:
    start_date = reading.start_date
    end_date = reading.end_date
    start_gran = reading.start_granularity
    end_gran = reading.end_granularity

    # Are start and end in the same day, year or month?
    same_day = False
    same_month = False
    same_year = False

    if start_date and end_date:
        if start_date.strftime('%Y') == end_date.strftime('%Y'):
            same_year = True
            if start_date.strftime('%m%Y') == end_date.strftime('%m%Y'):
                same_month = True
                if start_date.strftime('%d%m%Y') == end_date.strftime('%d%m%Y'):
                    same_day = True

    start_str = ''
    end_str = ''
    output = ''

    # Make some basic start and end strings, which we might use...

    if start_date:
        if start_gran == 3:
            start_str = start_date.strftime(full_format)
        elif start_gran == 4:
            start_str = start_date.strftime(month_year_format)
        else:
            start_str = start_date.strftime(year_format)

    if end_date:
        if end_gran == 3:
            end_str = end_date.strftime(full_format)
        elif end_gran == 4:
            end_str = end_date.strftime(month_year_format)
        else:
            end_str = end_date.strftime(year_format)

    # Now make the final strings we'll return:

    if start_date and end_date:

        # A default which will be overridden in many cases. This covers:
        # 1 February 2017 to 3 March 2018
        # 1 February 2017 to March 2018
        # 1 February 2017 to 2018
        # February 2017 to 3 March 2018
        # February 2017 to March 2018
        # February 2017 to 2018
        # 2017 to 3 March 2018
        # 2017 to March 2018
        # 2017 to 2018
        output = '{} to {}'.format(start_str, end_str)

        if (start_gran == 4 or end_gran == 4) and same_month:
            # Only have enough to output 'February 2017'.
            output = start_str

        elif (start_gran == 6 or end_gran == 6) and same_year:
            # Only have enough to output '2017'.
            output = start_str

        elif start_gran == 3:
            if end_gran == 3:
                if same_day:
                    # 1 February 2017
                    output = start_str

                elif same_month:
                    # 1–6 February 2017
                    output = '{}–{}'.format(
                                        start_date.strftime(day_format),
                                        end_str)
                elif same_year:
                    # 1 February to 3 March 2017
                    output = '{} to {}'.format(
                                        start_date.strftime(day_month_format),
                                        end_str)
            elif end_gran == 4:
                if same_year:
                    # 1 February to March 2017
                    output = '{} to {}'.format(
                                        start_date.strftime(day_month_format),
                                        end_str)
        elif start_gran == 4:
            if end_gran == 3:
                if same_year:
                    # February to 3 March 2017
                    output = '{} to {}'.format(
                                            start_date.strftime(month_format),
                                            end_str)
            elif end_gran == 4:
                if same_year:
                    # February to March 2017
                    output = '{} to {}'.format(
                                            start_date.strftime(month_format),
                                            end_str)
    elif end_date:
        # Only an end_date.
        if end_gran == 3:
            # Finished on 1 February 2017
            output = "Finished on {}".format(end_str)
        else:
            # Finished in February 2017
            # Finished in 2017
            output = "Finished in {}".format(end_str)

    else:
        # No end_date: the reading has started, but not ended.
        if start_gran == 3:
            # Started on 1 February 2017
            output = "Started on {}".format(start_str)

        else:
            # Started in February 2017
            # Started in 2017
            output = "Started in {}".format(start_str)

    return format_html(output)



# Generated by Django 2.0 on 2018-01-29 17:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0019_auto_20180127_1653'),
    ]

    operations = [
        migrations.AddField(
            model_name='venue',
            name='note',
            field=models.TextField(blank=True, help_text='Optional. Paragraphs will be surrounded with &lt;p&gt;&lt;/p&gt; tags. HTML allowed.'),
        ),
    ]



# Generated by Django 2.0 on 2018-01-26 09:09

from django.db import migrations


def forward(apps, schema_editor):
    """
    Copying data from the old `Event.movie` and `Event.play` ForeignKey fields
    into the new `Event.movies` and `Event.plays` ManyToManyFields.
    """

    Event = apps.get_model('spectator_events', 'Event')
    MovieSelection = apps.get_model('spectator_events', 'MovieSelection')
    PlaySelection = apps.get_model('spectator_events', 'PlaySelection')

    for event in Event.objects.all():
        if event.movie is not None:
            selection = MovieSelection(event=event, movie=event.movie)
            selection.save()

        if event.play is not None:
            selection = PlaySelection(event=event, play=event.play)
            selection.save()


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0016_add_movies_plays_m2ms_on_event'),
    ]

    operations = [
        migrations.RunPython(forward),
    ]



from django import template

from ..models import Event


register = template.Library()


@register.inclusion_tag('spectator_events/includes/event_list_tabs.html')
def event_list_tabs(counts, current_kind):
    """
    Displays the tabs to different event_list pages.

    `counts` is a dict of number of events for each kind, like:
        {'all': 30, 'gig': 12, 'movie': 18,}

    `current_kind` is the event kind that's active, if any. e.g. 'gig',
        'movie', etc.
    """
    return {
            'counts': counts,
            'current_kind': current_kind,
            # A list of all the kinds we might show tabs for, like
            # ['gig', 'movie', 'play', ...]
            'event_kinds': Event.get_kinds(),
            # A dict of data about each kind, keyed by kind ('gig') including
            # data about 'name', 'name_plural' and 'slug':
            'event_kinds_data': Event.get_kinds_data(),
        }


@register.simple_tag
def recent_events(num=10):
    """
    Returns a QuerySet of Events that happened recently.
    `num` is the number returned.
    """
    return Event.objects.select_related('venue').order_by('-date')[:num]


@register.inclusion_tag('spectator_events/includes/card_events.html')
def recent_events_card(num=10):
    """
    Displays Events that happened recently.
    `num` is the number returned.
    """
    return {
            'card_title': 'Recent events',
            'event_list': recent_events(num=num),
            }

@register.simple_tag
def day_events(date):
    """
    Returns a QuerySet of Events that happened on the supplied date.
    `date` is a date object.
    """
    return Event.objects.filter(date=date).select_related('venue')


@register.inclusion_tag('spectator_events/includes/card_events.html')
def day_events_card(date):
    """
    Displays Events that happened on the supplied date.
    `date` is a date object.
    """
    card_title = 'Events on {}'.format(date.strftime('%-d %b %Y'))
    return {
            'card_title': card_title,
            'event_list': day_events(date=date),
            }


@register.simple_tag
def events_years():
    """
    Returns a QuerySet of date objects, one for each year in which there are
    Events.
    """
    return Event.objects.dates('date', 'year')


@register.inclusion_tag('spectator_events/includes/card_years.html')
def events_years_card(current_year=None):
    """
    Displays a card showing all years in which we have Events, with a link to
    the archive pages.

    current_year is a date object representing the year we're displaying.
    """
    return {
            'current_year': current_year,
            'years': events_years(),
            }



from django.conf.urls import url

from .. import views


# Only the home page.
# This should be under the namespace 'spectator:core'.

app_name='core'

urlpatterns = [
    url(
        regex=r"^$",
        view=views.HomeView.as_view(),
        name='home'
    ),
]




# Generated by Django 2.0 on 2018-02-08 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spectator_events', '0030_movies_to_works'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='event',
            name='classicalworks',
        ),
        migrations.RemoveField(
            model_name='event',
            name='dancepieces',
        ),
        migrations.RemoveField(
            model_name='event',
            name='movies',
        ),
        migrations.RemoveField(
            model_name='event',
            name='plays',
        ),
        migrations.AddField(
            model_name='event',
            name='works',
            field=models.ManyToManyField(blank=True, through='spectator_events.WorkSelection', to='spectator_events.Work'),
        ),
    ]
